@@ -14,7 +14,9 @@
 int compile_pass (systemtap_session& s);
 int run_pass (systemtap_session& s);
 
-int make_tracequery(systemtap_session& s, std::string& name, const std::vector<std::string>& extra_headers);
+int make_tracequery(systemtap_session& s, std::string& name,
+                    const std::vector<std::string>& extra_headers);
+int make_typequery(systemtap_session& s, std::string& module);
 
 #endif // BUILDRUN_H
 
