@@ -174,6 +174,7 @@ find_script_hash (systemtap_session& s, const string& script, const hash &base)
   h.add(s.ignore_vmlinux);		// --ignore-vmlinux
   h.add(s.ignore_dwarf);		// --ignore-dwarf
   h.add(s.consult_symtab);		// --kelf, --kmap
+  h.add(s.skip_badvars);		// --skip-badvars
   if (!s.kernel_symtab_path.empty())	// --kmap
     {
       h.add(s.kernel_symtab_path);
@@ -272,4 +273,24 @@ find_tracequery_hash (systemtap_session& s)
   s.tracequery_path = hashdir + "/tracequery_" + result + ".ko";
 }
 
+
+void
+find_typequery_hash (systemtap_session& s, const string& name, string& module)
+{
+  hash h;
+  get_base_hash(s, h);
+
+  // Add the typequery name to distinguish the hash
+  h.add(name);
+
+  // Get the directory path to store our cached module
+  string result, hashdir;
+  h.result(result);
+  if (!create_hashdir(s, result, hashdir))
+    return;
+
+  module = hashdir + "/typequery_" + result
+    + (name[0] == 'k' ? ".ko" : ".so");
+}
+
 /* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
