@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2008 Red Hat Inc.
+// Copyright (C) 2005-2009 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -51,7 +51,7 @@ version ()
     << "SystemTap translator/driver "
     << "(version " << VERSION << "/" << dwfl_version (NULL)
     << " " << GIT_MESSAGE << ")" << endl
-    << "Copyright (C) 2005-2008 Red Hat, Inc. and others" << endl
+    << "Copyright (C) 2005-2009 Red Hat, Inc. and others" << endl
     << "This is free software; see the source for copying conditions." << endl;
 }
 
@@ -74,9 +74,16 @@ usage (systemtap_session& s, int exitcode)
     << endl
     << "Options:" << endl
     << "   --         end of translator options, script options follow" << endl
-    << "   -v         increase verbosity [" << s.verbose << "]" << endl
     << "   -h         show help" << endl
     << "   -V         show version" << endl
+    << "   -p NUM     stop after pass NUM 1-5, instead of " << s.last_pass << endl
+    << "              (parse, elaborate, translate, compile, run)" << endl
+    << "   -v         add verbosity to all passes" << endl
+    << "   --vp {N}+  add per-pass verbosity [";
+  for (unsigned i=0; i<5; i++)
+    clog << (s.perpass_verbose[i] <= 9 ? s.perpass_verbose[i] : 9);
+  clog 
+    << "]" << endl
     << "   -k         keep temporary directory" << endl
     << "   -u         unoptimized translation" << (s.unoptimized ? " [set]" : "") << endl
     << "   -w         suppress warnings" << (s.suppress_warnings ? " [set]" : "") << endl
@@ -84,11 +91,7 @@ usage (systemtap_session& s, int exitcode)
     << "   -P         prologue-searching for function probes"
     << (s.prologue_searching ? " [set]" : "") << endl
     << "   -b         bulk (percpu file) mode" << (s.bulk_mode ? " [set]" : "") << endl
-    << "   -s NUM     buffer size in megabytes, instead of "
-    << s.buffer_size << endl
-    << "   -p NUM     stop after pass NUM 1-5, instead of "
-    << s.last_pass << endl
-    << "              (parse, elaborate, translate, compile, run)" << endl
+    << "   -s NUM     buffer size in megabytes, instead of " << s.buffer_size << endl
     << "   -I DIR     look in DIR for additional .stp script files";
   if (s.include_path.size() == 0)
     clog << endl;
@@ -99,16 +102,19 @@ usage (systemtap_session& s, int exitcode)
   clog
     << "   -D NM=VAL  emit macro definition into generated C code" << endl
     << "   -R DIR     look in DIR for runtime, instead of" << endl
-    <<      "              " << s.runtime_path << endl
-    << "   -r RELEASE cross-compile to kernel RELEASE, instead of "
-    << s.kernel_release << endl
-    << "   -m MODULE  set probe module name, instead of "
-    << s.module_name << endl
-    << "   -o FILE    send output to file, instead of stdout" << endl
-    << "   -c CMD     start the probes, run CMD, and exit when it finishes"
-    << endl
+    << "              " << s.runtime_path << endl
+    << "   -r DIR     cross-compile to kernel with given build tree; or else" << endl
+    << "   -r RELEASE cross-compile to kernel /lib/modules/RELEASE/build, instead of" << endl
+    << "              " << s.kernel_build_tree << endl
+    << "   -m MODULE  set probe module name, instead of " << endl
+    << "              " << s.module_name << endl
+    << "   -o FILE    send script output to file, instead of stdout. This supports" << endl
+    << "              strftime(3) formats for FILE" << endl
+    << "   -c CMD     start the probes, run CMD, and exit when it finishes" << endl
     << "   -x PID     sets target() to PID" << endl
-    << "   -F         load module and start probes, then detach" << endl
+    << "   -F         run as on-file flight recorder with -o." << endl
+    << "              run as on-memory flight recorder without -o." << endl
+    << "   -S size[,n] set maximum of the size and the number of files." << endl
     << "   -d OBJECT  add unwind/symbol data for OBJECT file";
   if (s.unwindsym_modules.size() == 0)
     clog << endl;
@@ -131,6 +137,8 @@ usage (systemtap_session& s, int exitcode)
 #endif
   // Formerly present --ignore-{vmlinux,dwarf} options are for testsuite use
   // only, and don't belong in the eyesight of a plain user.
+    << "   --skip-badvars" << endl
+    << "              overlook context of bad $ variables" << endl
     << endl
     ;
 
@@ -184,13 +192,16 @@ printscript(systemtap_session& s, ostream& o)
             {
               o << pp;
               // Print the locals for -L mode only
-              if (s.unoptimized)
+              if (s.unoptimized) {
                 for (unsigned j=0; j<p->locals.size(); j++)
                   {
                     o << " ";
                     vardecl* v = p->locals[j];
                     v->printsig (o);
                   }
+                // Print arguments of probe if there
+                p->printargs(o);
+		}
               o << endl;
               seen.insert (pp);
             }
@@ -279,14 +290,9 @@ int pending_interrupts;
 extern "C"
 void handle_interrupt (int sig)
 {
-  if (pending_interrupts == 0)
-    kill (0, sig); // Forward signals to child processes if any.
-
+  kill_stap_spawn(sig);
   pending_interrupts ++;
-  // NB: the "2" below is intended to skip the effect of the self-induced
-  // deferred signal coming from the kill() above.
-
-  if (pending_interrupts > 2) // XXX: should be configurable? time-based?
+  if (pending_interrupts > 1) // XXX: should be configurable? time-based?
     {
       char msg[] = "Too many interrupts received, exiting.\n";
       int rc = write (2, msg, sizeof(msg)-1);
@@ -310,7 +316,7 @@ setup_signals (sighandler_t handler)
       sigaddset (&sa.sa_mask, SIGINT);
       sigaddset (&sa.sa_mask, SIGTERM);
     }
-  sa.sa_flags = 0;
+  sa.sa_flags = SA_RESTART;
 
   sigaction (SIGHUP, &sa, NULL);
   sigaction (SIGPIPE, &sa, NULL);
@@ -325,7 +331,6 @@ main (int argc, char * const argv [])
   string cmdline_script; // -e PROGRAM
   string script_file; // FILE
   bool have_script = false;
-  bool release_changed = false;
   bool save_module = false;
 
   // Initialize defaults
@@ -333,8 +338,10 @@ main (int argc, char * const argv [])
   struct utsname buf;
   (void) uname (& buf);
   s.kernel_release = string (buf.release);
+  s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
+
   s.architecture = string (buf.machine);
-  s.verbose = 0;
+  for (unsigned i=0; i<5; i++) s.perpass_verbose[i]=0;
   s.timing = false;
   s.guru_mode = false;
   s.bulk_mode = false;
@@ -351,6 +358,7 @@ main (int argc, char * const argv [])
   s.buffer_size = 0;
   s.last_pass = 5;
   s.module_name = "stap_" + stringify(getpid());
+  s.stapconf_name = "stapconf_" + stringify(getpid()) + ".h";
   s.output_file = ""; // -o FILE
   s.keep_tmpdir = false;
   s.cmd = "";
@@ -365,6 +373,7 @@ main (int argc, char * const argv [])
   s.ignore_vmlinux = false;
   s.ignore_dwarf = false;
   s.load_only = false;
+  s.skip_badvars = false;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
   if (s_p != NULL)
@@ -390,9 +399,10 @@ main (int argc, char * const argv [])
   if (create_dir(s.data_path.c_str()) == 1)
     {
       const char* e = strerror (errno);
-      cerr << "Warning: failed to create systemtap data directory (\""
-	   << s.data_path << "\"): " << e << endl;
-      cerr << "Disabling cache support." << endl;
+      if (! s.suppress_warnings)
+        cerr << "Warning: failed to create systemtap data directory (\""
+             << s.data_path << "\"): " << e
+             << ", disabling cache support." << endl;
       s.use_cache = false;
     }
 
@@ -402,9 +412,10 @@ main (int argc, char * const argv [])
       if (create_dir(s.cache_path.c_str()) == 1)
         {
 	  const char* e = strerror (errno);
-	  cerr << "Warning: failed to create cache directory (\""
-	       << s.cache_path << "\"): " << e << endl;
-	  cerr << "Disabling cache support." << endl;
+          if (! s.suppress_warnings)
+            cerr << "Warning: failed to create cache directory (\""
+                 << s.cache_path << "\"): " << e
+                 << ", disabling cache support." << endl;
 	  s.use_cache = false;
 	}
     }
@@ -413,6 +424,36 @@ main (int argc, char * const argv [])
   if (s_tc != NULL)
     s.tapset_compile_coverage = true;
 
+  const char* s_kr = getenv ("SYSTEMTAP_RELEASE");
+  if (s_kr != NULL) {
+    if (s_kr[0] == '/') // fully specified path
+      {
+        s.kernel_build_tree = s_kr;
+        string version_file_name = s.kernel_build_tree + "/include/config/kernel.release";
+        // The file include/config/kernel.release within the
+        // build tree is used to pull out the version information
+        ifstream version_file (version_file_name.c_str());
+        if (version_file.fail ())
+          {
+            cerr << "Missing " << version_file_name << endl;
+            exit(1);
+          }
+        else
+          {
+            char c;
+            s.kernel_release = "";
+            while (version_file.get(c) && c != '\n')
+              s.kernel_release.push_back(c);
+          }
+      }
+    else
+      {
+        s.kernel_release = string (s_kr);
+        s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
+      }
+  }
+
+
   while (true)
     {
       int long_opt;
@@ -420,15 +461,19 @@ main (int argc, char * const argv [])
 #define LONG_OPT_KMAP 2
 #define LONG_OPT_IGNORE_VMLINUX 3
 #define LONG_OPT_IGNORE_DWARF 4
+#define LONG_OPT_VERBOSE_PASS 5
+#define LONG_OPT_SKIP_BADVARS 6
       // NB: also see find_hash(), usage(), switch stmt below, stap.1 man page
       static struct option long_options[] = {
         { "kelf", 0, &long_opt, LONG_OPT_KELF },
         { "kmap", 2, &long_opt, LONG_OPT_KMAP },
         { "ignore-vmlinux", 0, &long_opt, LONG_OPT_IGNORE_VMLINUX },
         { "ignore-dwarf", 0, &long_opt, LONG_OPT_IGNORE_DWARF },
+	{ "skip-badvars", 0, &long_opt, LONG_OPT_SKIP_BADVARS },
+        { "vp", 1, &long_opt, LONG_OPT_VERBOSE_PASS },
         { NULL, 0, NULL, 0 }
       };
-      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:d:L:F",
+      int grc = getopt_long (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:uqwl:d:L:FS:",
                                                           long_options, NULL);
       if (grc < 0)
         break;
@@ -443,7 +488,8 @@ main (int argc, char * const argv [])
           break;
 
         case 'v':
-	  s.verbose ++;
+          for (unsigned i=0; i<5; i++)
+            s.perpass_verbose[i] ++;
 	  break;
 
         case 't':
@@ -542,8 +588,31 @@ main (int argc, char * const argv [])
           break;
 
         case 'r':
-          s.kernel_release = string (optarg);
-	  release_changed = true;
+          if (optarg[0] == '/') // fully specified path
+            {
+              s.kernel_build_tree = optarg;
+              string version_file_name = s.kernel_build_tree + "/include/config/kernel.release";
+              // The file include/config/kernel.release within the
+              // build tree is used to pull out the version information
+              ifstream version_file (version_file_name.c_str());
+              if (version_file.fail ())
+                {
+                  cerr << "Missing " << version_file_name << endl;
+                  usage (s, 1);
+                }
+              else 
+                {
+                  char c;
+                  s.kernel_release = "";
+                  while (version_file.get(c) && c != '\n')
+                    s.kernel_release.push_back(c);
+                }
+            }
+          else
+            {
+              s.kernel_release = string (optarg);
+              s.kernel_build_tree = "/lib/modules/" + s.kernel_release + "/build";
+            }
           break;
 
         case 'k':
@@ -588,6 +657,10 @@ main (int argc, char * const argv [])
 	  s.macros.push_back (string (optarg));
 	  break;
 
+	case 'S':
+	  s.size_option = string (optarg);
+	  break;
+
 	case 'q':
 	  s.tapset_compile_coverage = true;
 	  break;
@@ -642,6 +715,29 @@ main (int argc, char * const argv [])
 	    case LONG_OPT_IGNORE_DWARF:
 	      s.ignore_dwarf = true;
 	      break;
+	    case LONG_OPT_VERBOSE_PASS:
+              {
+                bool ok = true;
+                if (strlen(optarg) < 1 || strlen(optarg) > 5)
+                  ok = false;
+                if (ok)
+                  for (unsigned i=0; i<strlen(optarg); i++)
+                    if (isdigit (optarg[i]))
+                      s.perpass_verbose[i] += optarg[i]-'0';
+                    else
+                      ok = false;
+                
+                if (! ok)
+                  {
+                    cerr << "Invalid --vp argument: it takes 1 to 5 digits." << endl;
+                    usage (s, 1);
+                  }
+                // NB: we don't do this: s.last_pass = strlen(optarg);
+                break;
+              }
+	    case LONG_OPT_SKIP_BADVARS:
+	      s.skip_badvars = true;
+	      break;
             default:
               cerr << "Internal error parsing command arguments." << endl;
               usage(s, 1);
@@ -684,12 +780,15 @@ main (int argc, char * const argv [])
         s.kernel_symtab_path = string("/boot/System.map-") + s.kernel_release;
     }
 
-  if (s.last_pass > 4 && release_changed)
-    {
-      if (s.verbose)
-        cerr << "Warning: changing last pass to 4 since cross-compiling" << endl;
-      s.last_pass = 4;
-    }
+  // Warn in case the target kernel release doesn't match the running one.
+  if (s.last_pass > 4 &&
+      (string(buf.release) != s.kernel_release ||
+       string(buf.machine) != s.architecture))
+   {
+     if(! s.suppress_warnings)
+       cerr << "WARNING: kernel release/architecture mismatch with host forces last-pass 4." << endl;
+     s.last_pass = 4;
+   }
 
   for (int i = optind; i < argc; i++)
     {
@@ -720,6 +819,9 @@ main (int argc, char * const argv [])
     }
 
   int rc = 0;
+  
+  // PASS 0: setting up
+  s.verbose = s.perpass_verbose[0];
 
   // For PR1477, we used to override $PATH and $LC_ALL and other stuff
   // here.  We seem to use complete pathnames in
@@ -910,9 +1012,9 @@ main (int argc, char * const argv [])
            << endl;
     }
 
-  if (rc)
+  if (rc && !s.listing_mode)
     cerr << "Pass 1: parse failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 1' option."
          << endl;
 
   if (rc || s.last_pass == 1 || pending_interrupts) goto cleanup;
@@ -921,6 +1023,7 @@ main (int argc, char * const argv [])
   gettimeofday (&tv_before, NULL);
 
   // PASS 2: ELABORATION
+  s.verbose = s.perpass_verbose[1];
   rc = semantic_pass (s);
 
   if (s.listing_mode || (rc == 0 && s.last_pass == 2))
@@ -937,9 +1040,9 @@ main (int argc, char * const argv [])
                       << TIMESPRINT
                       << endl;
 
-  if (rc)
+  if (rc && !s.listing_mode)
     cerr << "Pass 2: analysis failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 01' option."
          << endl;
   // Generate hash.  There isn't any point in generating the hash
   // if last_pass is 2, since we'll quit before using it.
@@ -975,7 +1078,7 @@ main (int argc, char * const argv [])
   if (rc || s.listing_mode || s.last_pass == 2 || pending_interrupts) goto cleanup;
 
   // PASS 3: TRANSLATION
-
+  s.verbose = s.perpass_verbose[2];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
 
@@ -998,13 +1101,13 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 3: translation failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 001' option."
          << endl;
 
   if (rc || s.last_pass == 3 || pending_interrupts) goto cleanup;
 
   // PASS 4: COMPILATION
-
+  s.verbose = s.perpass_verbose[3];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   rc = compile_pass (s);
@@ -1026,14 +1129,19 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 4: compilation failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 0001' option."
          << endl;
   else
     {
       // Update cache. Cache cleaning is kicked off at the end of this function.
       if (s.use_cache)
-	add_to_cache(s);
+        add_to_cache(s);
 
+      // We may need to save the module in $CWD if the cache was
+      // inaccessible for some reason.
+      if (! s.use_cache && s.last_pass == 4)
+        save_module = true;
+      
       // Copy module to the current directory.
       if (save_module && !pending_interrupts)
         {
@@ -1054,6 +1162,7 @@ main (int argc, char * const argv [])
 
   // PASS 5: RUN
 pass_5:
+  s.verbose = s.perpass_verbose[4];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   // NB: this message is a judgement call.  The other passes don't emit
@@ -1069,11 +1178,12 @@ pass_5:
 
   if (rc)
     cerr << "Pass 5: run failed.  "
-         << "Try again with more '-v' (verbose) options."
+         << "Try again with another '--vp 00001' option."
          << endl;
 
   // if (rc) goto cleanup;
 
+  // PASS 6: cleaning up
  cleanup:
 
   // update the database information
@@ -1101,7 +1211,7 @@ pass_5:
           string cleanupcmd = "rm -rf ";
           cleanupcmd += s.tmpdir;
           if (s.verbose>1) clog << "Running " << cleanupcmd << endl;
-	  int status = system (cleanupcmd.c_str());
+	  int status = stap_system (cleanupcmd.c_str());
 	  if (status != 0 && s.verbose>1)
 	    clog << "Cleanup command failed, status: " << status << endl;
         }
@@ -1109,3 +1219,5 @@ pass_5:
 
   return (rc||pending_interrupts) ? EXIT_FAILURE : EXIT_SUCCESS;
 }
+
+/* vim: set sw=2 ts=8 cino=>4,n-2,{2,^-2,t0,(0,u0,w1,M1 : */
