@@ -29,11 +29,12 @@
 	    & (((__typeof (base)) 1 << (nbits)) - 1))
 
 #define store_bitfield(target, base, higherbits, nbits)			      \
-  target = (target							      \
-	    &~ ((((__typeof (base)) 1 << (nbits)) - 1)			      \
-		<< (sizeof (base) * 8 - (higherbits) - (nbits)))	      \
-	    | ((__typeof (base)) (base)					      \
-	       << (sizeof (base) * 8 - (higherbits) - (nbits))))
+  target = ((target							      \
+	     &~ ((((__typeof (target)) 1 << (nbits)) - 1)		      \
+		 << (sizeof (target) * 8 - (higherbits) - (nbits))))	      \
+	    | ((((__typeof (target)) (base))				      \
+		& (((__typeof (target)) 1 << (nbits)) - 1))		      \
+	       << (sizeof (target) * 8 - (higherbits) - (nbits))))
 
 
 /* Given a DWARF register number, fetch its intptr_t (long) value from the
@@ -62,6 +63,10 @@
    must work right for kernel addresses, and can use whatever existing
    machine-specific kernel macros are convenient.  */
 
+#if STP_SKIP_BADVARS
+#define DEREF_FAULT(addr) ({0; })
+#define STORE_DEREF_FAULT(addr) ({0; })
+#else
 #define DEREF_FAULT(addr) ({						    \
     snprintf(c->error_buffer, sizeof(c->error_buffer),			    \
       "kernel read fault at 0x%p (%s)", (void *)(intptr_t)(addr), #addr);   \
@@ -75,7 +80,7 @@
     c->last_error = c->error_buffer;					    \
     goto deref_fault;							    \
     })
-
+#endif
 
 #if defined (STAPCONF_X86_UNIREGS) && defined (__i386__)
 
@@ -113,16 +118,13 @@
    kernel mode, it is not saved in the trap frame (struct pt_regs).
    The `esp' (and `xss') fields are valid only for a user-mode trap.
    For a kernel mode trap, the interrupted state's esp is actually an
-   address inside where the `struct pt_regs' on the kernel trap stack points.
-
-   For now we assume all traps are from kprobes in kernel-mode code.
-   For extra paranoia, could do BUG_ON((regs->xcs & 3) == 3).  */
+   address inside where the `struct pt_regs' on the kernel trap stack points. */
 
 #define dwarf_register_0(regs)	regs->eax
 #define dwarf_register_1(regs)	regs->ecx
 #define dwarf_register_2(regs)	regs->edx
 #define dwarf_register_3(regs)	regs->ebx
-#define dwarf_register_4(regs)	((long) &regs->esp)
+#define dwarf_register_4(regs)	(user_mode(regs) ? regs->esp : (long)&regs->esp)
 #define dwarf_register_5(regs)	regs->ebp
 #define dwarf_register_6(regs)	regs->esi
 #define dwarf_register_7(regs)	regs->edi
@@ -189,28 +191,29 @@
  */
 
 #define kread(ptr) ({ \
-        typeof(*(ptr)) _v; \
-        if (probe_kernel_read((void *)&_v, (void *)(ptr), sizeof(*(ptr)))) \
-            DEREF_FAULT(ptr); \
+        typeof(*(ptr)) _v = 0; \
+        if (lookup_bad_addr((unsigned long)(ptr)) || \
+            probe_kernel_read((void *)&_v, (void *)(ptr), sizeof(*(ptr)))) \
+          DEREF_FAULT(ptr); \
         _v; \
     })
 
 #define kwrite(ptr, value) ({ \
         typeof(*(ptr)) _v; \
         _v = (typeof(*(ptr)))(value); \
-        if (probe_kernel_write((void *)(ptr), (void *)&_v, sizeof(*(ptr)))) \
-            STORE_DEREF_FAULT(ptr); \
+        if (lookup_bad_addr((unsigned long)addr) || \
+            probe_kernel_write((void *)(ptr), (void *)&_v, sizeof(*(ptr)))) \
+          STORE_DEREF_FAULT(ptr); \
     })
 
 #define deref(size, addr) ({ \
-    intptr_t _i; \
+    intptr_t _i = 0; \
     switch (size) { \
       case 1: _i = kread((u8 *)(addr)); break; \
       case 2: _i = kread((u16 *)(addr)); break; \
       case 4: _i = kread((u32 *)(addr)); break; \
       case 8: _i = kread((u64 *)(addr)); break; \
       default: __deref_bad(); \
-      /* uninitialized _i should also be caught by -Werror */ \
     } \
     _i; \
   })
@@ -236,31 +239,37 @@ extern void __store_deref_bad(void);
   ({									      \
     int _bad = 0;							      \
     u8 _b; u16 _w; u32 _l;	                                              \
-    intptr_t _v;							      \
-    switch (size)							      \
-      {									      \
-      case 1: __get_user_asm(_b,addr,_bad,"b","b","=q",1); _v = _b; break;    \
-      case 2: __get_user_asm(_w,addr,_bad,"w","w","=r",1); _v = _w; break;    \
-      case 4: __get_user_asm(_l,addr,_bad,"l","","=r",1); _v = _l; break;     \
-      default: _v = __get_user_bad();					      \
-      }									      \
-    if (_bad)								      \
-      DEREF_FAULT(addr);							      \
+    intptr_t _v = 0;							      \
+    if (lookup_bad_addr((unsigned long)addr))                                 \
+      _bad = 1;                                                               \
+    else                                                                      \
+      switch (size)                                                           \
+        {                                                                     \
+        case 1: __get_user_asm(_b,addr,_bad,"b","b","=q",1); _v = _b; break;  \
+        case 2: __get_user_asm(_w,addr,_bad,"w","w","=r",1); _v = _w; break;  \
+        case 4: __get_user_asm(_l,addr,_bad,"l","","=r",1); _v = _l; break;   \
+        default: _v = __get_user_bad();                                       \
+        }                                                                     \
+    if (_bad)                                                                 \
+      DEREF_FAULT(addr);						      \
     _v;									      \
   })
 
 #define store_deref(size, addr, value)					      \
   ({									      \
     int _bad = 0;							      \
-    switch (size)							      \
-      {									      \
-      case 1: __put_user_asm(((u8)(value)),addr,_bad,"b","b","iq",1); break;  \
-      case 2: __put_user_asm(((u16)(value)),addr,_bad,"w","w","ir",1); break; \
-      case 4: __put_user_asm(((u32)(value)),addr,_bad,"l","k","ir",1); break; \
-      default: __put_user_bad();					      \
-      }									      \
-    if (_bad)								      \
-      STORE_DEREF_FAULT(addr);							      \
+    if (lookup_bad_addr((unsigned long)addr))                                 \
+      _bad = 1;                                                               \
+    else                                                                      \
+      switch (size)                                                           \
+        {                                                                     \
+        case 1: __put_user_asm(((u8)(value)),addr,_bad,"b","b","iq",1); break;\
+        case 2: __put_user_asm(((u16)(value)),addr,_bad,"w","w","ir",1); break;\
+        case 4: __put_user_asm(((u32)(value)),addr,_bad,"l","k","ir",1); break;\
+        default: __put_user_bad();                                            \
+        }                                                                     \
+    if (_bad)                                                                 \
+      STORE_DEREF_FAULT(addr);                                                \
   })
 
 
@@ -270,15 +279,18 @@ extern void __store_deref_bad(void);
   ({									      \
     int _bad = 0;							      \
     u8 _b; u16 _w; u32 _l; u64 _q;					      \
-    intptr_t _v;							      \
-    switch (size)							      \
-      {									      \
-      case 1: __get_user_asm(_b,addr,_bad,"b","b","=q",1); _v = _b; break;    \
-      case 2: __get_user_asm(_w,addr,_bad,"w","w","=r",1); _v = _w; break;    \
-      case 4: __get_user_asm(_l,addr,_bad,"l","","=r",1); _v = _l; break;     \
-      case 8: __get_user_asm(_q,addr,_bad,"q","","=r",1); _v = _q; break;     \
-      default: _v = __get_user_bad();					      \
-      }									      \
+    intptr_t _v = 0;							      \
+    if (lookup_bad_addr((unsigned long)addr))                                 \
+      _bad = 1;                                                               \
+    else                                                                      \
+      switch (size)                                                           \
+        {                                                                     \
+        case 1: __get_user_asm(_b,addr,_bad,"b","b","=q",1); _v = _b; break;  \
+        case 2: __get_user_asm(_w,addr,_bad,"w","w","=r",1); _v = _w; break;  \
+        case 4: __get_user_asm(_l,addr,_bad,"l","","=r",1); _v = _l; break;   \
+        case 8: __get_user_asm(_q,addr,_bad,"q","","=r",1); _v = _q; break;   \
+        default: _v = __get_user_bad();                                       \
+        }                                                                     \
     if (_bad)								      \
       DEREF_FAULT(addr);							      \
     _v;									      \
@@ -287,14 +299,17 @@ extern void __store_deref_bad(void);
 #define store_deref(size, addr, value)					      \
   ({									      \
     int _bad = 0;							      \
-    switch (size)							      \
-      {									      \
-      case 1: __put_user_asm(((u8)(value)),addr,_bad,"b","b","iq",1); break;  \
-      case 2: __put_user_asm(((u16)(value)),addr,_bad,"w","w","ir",1); break; \
-      case 4: __put_user_asm(((u32)(value)),addr,_bad,"l","k","ir",1); break; \
-      case 8: __put_user_asm(((u64)(value)),addr,_bad,"q","","Zr",1); break;  \
-      default: __put_user_bad();					      \
-      }									      \
+    if (lookup_bad_addr((unsigned long)addr))                                 \
+      _bad = 1;                                                               \
+    else                                                                      \
+      switch (size)                                                           \
+        {                                                                     \
+        case 1: __put_user_asm(((u8)(value)),addr,_bad,"b","b","iq",1); break; \
+        case 2: __put_user_asm(((u16)(value)),addr,_bad,"w","w","ir",1); break;\
+        case 4: __put_user_asm(((u32)(value)),addr,_bad,"l","k","ir",1); break;\
+        case 8: __put_user_asm(((u64)(value)),addr,_bad,"q","","Zr",1); break; \
+        default: __put_user_bad();                                      \
+        }                                                               \
     if (_bad)								      \
       STORE_DEREF_FAULT(addr);							      \
   })
@@ -304,13 +319,16 @@ extern void __store_deref_bad(void);
   ({									\
      int _bad = 0;							\
      intptr_t _v=0;							\
-	switch (size){							\
-	case 1: __get_user_size(_v, addr, 1, _bad); break; 		\
-	case 2: __get_user_size(_v, addr, 2, _bad); break;  		\
-	case 4: __get_user_size(_v, addr, 4, _bad); break;  		\
-	case 8: __get_user_size(_v, addr, 8, _bad); break;  		\
-	default: __get_user_unknown(); break;				\
-	}								\
+     if (lookup_bad_addr((unsigned long)addr))                          \
+       _bad = 1;                                                        \
+     else                                                               \
+       switch (size){							\
+       case 1: __get_user_size(_v, addr, 1, _bad); break; 		\
+       case 2: __get_user_size(_v, addr, 2, _bad); break;  		\
+       case 4: __get_user_size(_v, addr, 4, _bad); break;  		\
+       case 8: __get_user_size(_v, addr, 8, _bad); break;  		\
+       default: __get_user_unknown(); break;				\
+       }								\
     if (_bad)  								\
 	DEREF_FAULT(addr);						\
      _v;								\
@@ -319,13 +337,16 @@ extern void __store_deref_bad(void);
 #define store_deref(size, addr, value)					\
   ({									\
     int _bad=0;								\
-	switch (size){							\
-	case 1: __put_user_size(value, addr, 1, _bad); break;		\
-	case 2: __put_user_size(value, addr, 2, _bad); break;		\
-	case 4: __put_user_size(value, addr, 4, _bad); break;		\
-	case 8: __put_user_size(value, addr, 8, _bad); break;		\
-	default: __put_user_unknown(); break;				\
-	}								\
+    if (lookup_bad_addr((unsigned long)addr))                           \
+      _bad = 1;                                                         \
+    else                                                                \
+      switch (size){							\
+      case 1: __put_user_size(value, addr, 1, _bad); break;		\
+      case 2: __put_user_size(value, addr, 2, _bad); break;		\
+      case 4: __put_user_size(value, addr, 4, _bad); break;		\
+      case 8: __put_user_size(value, addr, 8, _bad); break;		\
+      default: __put_user_unknown(); break;				\
+      }                                                                 \
     if (_bad)								\
 	   STORE_DEREF_FAULT(addr);						\
    })
@@ -375,31 +396,37 @@ extern void __store_deref_bad(void);
 #define deref(size, addr)						      \
   ({									      \
     int _bad = 0;							      \
-    intptr_t _v;							      \
-    switch (size)							      \
-      {									      \
-      case 1: __stp_get_user_asm(_v,addr,_bad,"lbz"); break;		      \
-      case 2: __stp_get_user_asm(_v,addr,_bad,"lhz"); break;		      \
-      case 4: __stp_get_user_asm(_v,addr,_bad,"lwz"); break;		      \
-      case 8: __stp_get_user_asm(_v,addr,_bad,"ld"); break;			      \
-      default: _v = __get_user_bad();					      \
-      }									      \
+    intptr_t _v = 0;							      \
+    if (lookup_bad_addr((unsigned long)addr))                                 \
+      _bad = 1;                                                               \
+    else                                                                      \
+      switch (size)                                                           \
+        {                                                                     \
+        case 1: __stp_get_user_asm(_v,addr,_bad,"lbz"); break;                \
+        case 2: __stp_get_user_asm(_v,addr,_bad,"lhz"); break;                \
+        case 4: __stp_get_user_asm(_v,addr,_bad,"lwz"); break;                \
+        case 8: __stp_get_user_asm(_v,addr,_bad,"ld"); break;                 \
+        default: _v = __get_user_bad();                                       \
+        }                                                                     \
     if (_bad)								      \
-      DEREF_FAULT(addr);							      \
+      DEREF_FAULT(addr);						      \
     _v;									      \
   })
 
 #define store_deref(size, addr, value)					      \
   ({									      \
     int _bad = 0;							      \
-    switch (size)							      \
-      {									      \
-      case 1: __stp_put_user_asm(((u8)(value)),addr,_bad,"stb"); break;     \
-      case 2: __stp_put_user_asm(((u16)(value)),addr,_bad,"sth"); break;    \
-      case 4: __stp_put_user_asm(((u32)(value)),addr,_bad,"stw"); break;    \
-      case 8: __stp_put_user_asm(((u64)(value)),addr,_bad, "std"); break;         \
-      default: __put_user_bad();					      \
-      }									      \
+    if (lookup_bad_addr((unsigned long)addr))                                 \
+      _bad = 1;                                                               \
+    else                                                                      \
+      switch (size)                                                           \
+        {                                                                     \
+        case 1: __stp_put_user_asm(((u8)(value)),addr,_bad,"stb"); break;     \
+        case 2: __stp_put_user_asm(((u16)(value)),addr,_bad,"sth"); break;    \
+        case 4: __stp_put_user_asm(((u32)(value)),addr,_bad,"stw"); break;    \
+        case 8: __stp_put_user_asm(((u64)(value)),addr,_bad, "std"); break;   \
+        default: __put_user_bad();                                            \
+        }                                                                     \
     if (_bad)								      \
       STORE_DEREF_FAULT(addr);							      \
   })
@@ -544,12 +571,15 @@ extern void __store_deref_bad(void);
   ({									\
      int _bad = 0;							\
      intptr_t _v=0;							\
-	switch (size){							\
-	case 1: __stp_get_user_asm_byte(_v, addr, _bad); break; 	\
-	case 2: __stp_get_user_asm_half(_v, addr, _bad); break; 	\
-	case 4: __stp_get_user_asm_word(_v, addr, _bad); break; 	\
-	default: __get_user_bad(); break;				\
-	}								\
+     if (lookup_bad_addr((unsigned long)addr))                          \
+      _bad = 1;                                                         \
+    else                                                                \
+      switch (size){							\
+      case 1: __stp_get_user_asm_byte(_v, addr, _bad); break;           \
+      case 2: __stp_get_user_asm_half(_v, addr, _bad); break;           \
+      case 4: __stp_get_user_asm_word(_v, addr, _bad); break;           \
+      default: __get_user_bad(); break;                                 \
+      }                                                                 \
     if (_bad)  								\
 	DEREF_FAULT(addr);						\
      _v;								\
@@ -558,13 +588,16 @@ extern void __store_deref_bad(void);
 #define store_deref(size, addr, value)					\
   ({									\
     int _bad=0;								\
-	switch (size){							\
-	case 1: __stp_put_user_asm_byte(value, addr, _bad); break;	\
-	case 2: __stp_put_user_asm_half(value, addr, _bad); break;	\
-	case 4: __stp_put_user_asm_word(value, addr, _bad); break;	\
-	case 8: __stp_put_user_asm_dword(value, addr, _bad); break;	\
-	default: __put_user_bad(); break;				\
-	}								\
+    if (lookup_bad_addr((unsigned long)addr))                           \
+      _bad = 1;                                                         \
+    else                                                                \
+      switch (size){							\
+      case 1: __stp_put_user_asm_byte(value, addr, _bad); break;	\
+      case 2: __stp_put_user_asm_half(value, addr, _bad); break;	\
+      case 4: __stp_put_user_asm_word(value, addr, _bad); break;	\
+      case 8: __stp_put_user_asm_dword(value, addr, _bad); break;	\
+      default: __put_user_bad(); break;                                 \
+      }                                                                 \
     if (_bad)								\
 	   STORE_DEREF_FAULT(addr);						\
    })
@@ -627,28 +660,31 @@ extern void __store_deref_bad(void);
 	u8 _b; u16 _w; u32 _l; u64 _q;				\
 	int _bad = 0;						\
 	intptr_t _v = 0;					\
-	switch (size) {						\
-	case 1: {						\
+        if (lookup_bad_addr((unsigned long)addr))               \
+          _bad = 1;                                             \
+        else                                                    \
+          switch (size) {		                	\
+          case 1: {                                             \
 		__stp_get_asm(_b, addr, _bad, 1);		\
 		_v = _b;					\
 		break;						\
-		};						\
-	case 2: {						\
+          };                                                    \
+          case 2: {						\
 		__stp_get_asm(_w, addr, _bad, 2);		\
 		_v = _w;					\
 		break;						\
-		};						\
-	case 4: {						\
+          };                                                    \
+          case 4: {						\
 		__stp_get_asm(_l, addr, _bad, 4);		\
 		_v = _l;					\
 		break;						\
-		};						\
-	case 8: {						\
+          };                                                    \
+          case 8: {						\
 		__stp_get_asm(_q, addr, _bad, 8);		\
 		_v = _q;					\
 		break;						\
-		};						\
-	default:						\
+          };                                                    \
+          default:						\
 		_bad = -EFAULT;					\
 	}							\
 	if (_bad)						\
@@ -660,12 +696,17 @@ extern void __store_deref_bad(void);
 ({                                                              \
         int _bad = 0;                                           \
 	int i;							\
-        for(i=0;i<size;i++){                                    \
-		__stp_put_asm((u8)(value>>((size-i-1)*8)&0xff),	\
-                             (u64)addr+i,_bad); 		\
-		if (_bad)					\
-			STORE_DEREF_FAULT(addr);			\
-        }                                                       \
+        if (lookup_bad_addr((unsigned long)addr))               \
+          _bad = 1;                                             \
+        else                                                    \
+          for(i=0;i<size;i++){                                  \
+            __stp_put_asm((u8)(value>>((size-i-1)*8)&0xff),     \
+                          (u64)addr+i,_bad);                    \
+            if (_bad)                                           \
+              break;                                            \
+          }                                                     \
+        if (_bad)                                               \
+          STORE_DEREF_FAULT(addr);                              \
 })
 
 
