@@ -14,25 +14,19 @@
 
 #include "stat-common.c"
 
-
 static void _stp_map_print_histogram (MAP map, stat *sd)
 {
 	_stp_stat_print_histogram (&map->hist, sd);
 }
 
-static MAP _stp_map_new_hstat_log (unsigned max_entries, int key_size, int buckets)
+static MAP _stp_map_new_hstat_log (unsigned max_entries, int key_size)
 {
 	/* add size for buckets */
-	int size = buckets * sizeof(int64_t) + sizeof(stat);
+	int size = HIST_LOG_BUCKETS * sizeof(int64_t) + sizeof(stat);
 	MAP m = _stp_map_new (max_entries, STAT, key_size, size);
 	if (m) {
 		m->hist.type = HIST_LOG;
-		m->hist.buckets = buckets;
-		if (buckets < 1 || buckets > 64) {
-			_stp_warn("histogram: Bad number of buckets.  Must be between 1 and 64\n");
-			m->hist.type = HIST_NONE;
-			return m;
-		}
+		m->hist.buckets = HIST_LOG_BUCKETS;
 	}
 	return m;
 }
@@ -41,12 +35,13 @@ static MAP _stp_map_new_hstat_linear (unsigned max_entries, int ksize, int start
 {
 	MAP m;
 	int size;
-	int buckets = (stop - start) / interval;
-	if ((stop - start) % interval) buckets++;
-
+	int buckets = _stp_stat_calc_buckets(stop, start, interval);
+	if (!buckets)
+		return NULL;
+	
         /* add size for buckets */
 	size = buckets * sizeof(int64_t) + sizeof(stat);
-
+	
 	m = _stp_map_new (max_entries, STAT, ksize, size);
 	if (m) {
 		m->hist.type = HIST_LINEAR;
@@ -54,32 +49,27 @@ static MAP _stp_map_new_hstat_linear (unsigned max_entries, int ksize, int start
 		m->hist.stop = stop;
 		m->hist.interval = interval;
 		m->hist.buckets = buckets;
-		if (m->hist.buckets <= 0) {
-			_stp_warn("histogram: bad stop, start and/or interval\n");
-			m->hist.type = HIST_NONE;
-			return m;
-		}
-		
 	}
 	return m;
 }
 
-static MAP _stp_pmap_new_hstat_linear (unsigned max_entries, int ksize, int start, int stop, int interval)
+static PMAP _stp_pmap_new_hstat_linear (unsigned max_entries, int ksize, int start, int stop, int interval)
 {
-	MAP map;
+	PMAP pmap;
 	int size;
-	int buckets = (stop - start) / interval;
-	if ((stop - start) % interval) buckets++;
+	int buckets = _stp_stat_calc_buckets(stop, start, interval);
+	if (!buckets)
+		return NULL;
 
         /* add size for buckets */
 	size = buckets * sizeof(int64_t) + sizeof(stat);
 
-	map = _stp_pmap_new (max_entries, STAT, ksize, size);
-	if (map) {
+	pmap = _stp_pmap_new (max_entries, STAT, ksize, size);
+	if (pmap) {
 		int i;
 		MAP m;
-		for_each_cpu(i) {
-			m = _stp_per_cpu_ptr (map, i);
+		stp_for_each_cpu(i) {
+			m = (MAP)per_cpu_ptr (pmap->map, i);
 			m->hist.type = HIST_LINEAR;
 			m->hist.start = start;
 			m->hist.stop = stop;
@@ -87,33 +77,33 @@ static MAP _stp_pmap_new_hstat_linear (unsigned max_entries, int ksize, int star
 			m->hist.buckets = buckets;
 		}
 		/* now set agg map  params */
-		m = _stp_percpu_dptr(map);
+		m = &pmap->agg;
 		m->hist.type = HIST_LINEAR;
 		m->hist.start = start;
 		m->hist.stop = stop;
 		m->hist.interval = interval;
 		m->hist.buckets = buckets;
 	}
-	return map;
+	return pmap;
 }
 
-static MAP _stp_pmap_new_hstat_log (unsigned max_entries, int key_size, int buckets)
+static PMAP _stp_pmap_new_hstat_log (unsigned max_entries, int key_size)
 {
 	/* add size for buckets */
-	int size = buckets * sizeof(int64_t) + sizeof(stat);
-	MAP map = _stp_pmap_new (max_entries, STAT, key_size, size);
-	if (map) {
+	int size = HIST_LOG_BUCKETS * sizeof(int64_t) + sizeof(stat);
+	PMAP pmap = _stp_pmap_new (max_entries, STAT, key_size, size);
+	if (pmap) {
 		int i;
 		MAP m;
-		for_each_cpu(i) {
-			m = _stp_per_cpu_ptr (map, i);
+		stp_for_each_cpu(i) {
+			m = (MAP)per_cpu_ptr (pmap->map, i);
 			m->hist.type = HIST_LOG;
-			m->hist.buckets = buckets;
+			m->hist.buckets = HIST_LOG_BUCKETS;
 		}
 		/* now set agg map  params */
-		m = _stp_percpu_dptr(map);
+		m = &pmap->agg;
 		m->hist.type = HIST_LOG;
-		m->hist.buckets = buckets;
+		m->hist.buckets = HIST_LOG_BUCKETS;
 	}
-	return map;
+	return pmap;
 }
