@@ -23,8 +23,8 @@ static void __stp_show_stack_sym(struct unw_frame_info *info, void *arg)
                 if (skip){
 			if (ip == REG_IP(regs))
 				skip = 0;
-                        else continue;
-                }
+			continue;
+		}
 		_stp_print_char(' ');
 		_stp_symbol_print(ip);
 		_stp_print_char('\n');
@@ -48,7 +48,8 @@ static void __stp_show_stack_addr(struct unw_frame_info *info, void *arg)
 	} while (unw_unwind(info) >= 0);
 }
 
-static void __stp_stack_print (struct pt_regs *regs, int verbose, int levels)
+static void __stp_stack_print (struct pt_regs *regs, int verbose, int levels,
+                               struct task_struct *tsk)
 {
 	unsigned long *stack = (unsigned long *)&REG_SP(regs);
 	struct dump_para para;
