@@ -144,7 +144,7 @@ void start_cmd(void)
       {
         /* The user must have used a shell metacharacter, thinking that
            we use system(3) to evaluate 'stap -c CMD'.  We could generate
-           an error message ... but let's just do what the user meant.  
+           an error message ... but let's just do what the user meant.
            rhbz 467652. */
         sh_c_argv[0] = "sh";
         sh_c_argv[1] = "-c";
@@ -153,15 +153,15 @@ void start_cmd(void)
       }
     else
       {
-        switch (rc) 
+        switch (rc)
           {
-          case 0: 
+          case 0:
             break;
           case WRDE_SYNTAX:
             _err ("wordexp: syntax error (unmatched quotes?) in -c COMMAND\n");
             _exit(1);
           default:
-            _err ("wordexp: parsing error (%d)\n", rc); 
+            _err ("wordexp: parsing error (%d)\n", rc);
             _exit (1);
           }
         if (words.we_wordc < 1) { _err ("empty -c COMMAND"); _exit (1); }
@@ -239,7 +239,6 @@ void system_cmd(char *cmd)
   if ((pid = fork()) < 0) {
     _perr("fork");
   } else if (pid == 0) {
-    setpriority(PRIO_PROCESS, 0, 0);
     if (execl("/bin/sh", "sh", "-c", cmd, NULL) < 0)
       perr("%s", cmd);
     _exit(1);
@@ -319,6 +318,41 @@ int init_stapio(void)
   if (target_cmd)
     start_cmd();
 
+  /* Run in background */
+  if (daemon_mode) {
+    pid_t pid;
+    int ret;
+    dbug(2, "daemonizing stapio\n");
+
+    /* daemonize */
+    ret = daemon(0, 1); /* don't close stdout at this time. */
+    if (ret) {
+      err("Failed to daemonize stapio\n");
+      return -1;
+    }
+
+    /* change error messages to syslog. */
+    switch_syslog("stapio");
+
+    /* show new pid */
+    pid = getpid();
+    fprintf(stdout, "%d\n", pid);
+    fflush(stdout);
+
+    /* redirect all outputs to /dev/null */
+    ret = open("/dev/null", O_RDWR);
+    if (ret < 0) {
+      err("Failed to open /dev/null\n");
+      return -1;
+    }
+    close(STDIN_FILENO);
+    close(STDOUT_FILENO);
+    close(STDERR_FILENO);
+    dup2(ret, STDOUT_FILENO);
+    dup2(ret, STDERR_FILENO);
+    close(ret);
+  }
+
   return 0;
 }
 
@@ -357,11 +391,65 @@ void cleanup_and_exit(int detach)
   if (detach) {
     err("\nDisconnecting from systemtap module.\n" "To reconnect, type \"staprun -A %s\"\n", modname);
   } else {
+    const char *staprun = getenv ("SYSTEMTAP_STAPRUN") ?: BINDIR "/staprun";
+#define BUG9788_WORKAROUND
+#ifndef BUG9788_WORKAROUND
     dbug(2, "removing %s\n", modname);
-    if (execl(BINDIR "/staprun", "staprun", "-d", modname, NULL) < 0) {
-      perror(modname);
+    if (execlp(staprun, basename (staprun), "-d", modname, NULL) < 0) {
+      if (errno == ENOEXEC) {
+	char *cmd;
+	if (asprintf(&cmd, "%s -d '%s'", staprun, modname) > 0)
+	  execl("/bin/sh", "sh", "-c", cmd, NULL);
+	free(cmd);
+      }
+      perror(staprun);
       _exit(1);
     }
+#else
+    pid_t pid;
+    int rstatus;
+    struct sigaction sa;
+
+    dbug(2, "removing %s\n", modname);
+
+    // So that waitpid() below will work correctly, we need to clear
+    // out our SIGCHLD handler.
+    sigemptyset(&sa.sa_mask);
+    sa.sa_flags = 0;
+    sa.sa_handler = SIG_DFL;
+    sigaction(SIGCHLD, &sa, NULL);
+
+    pid = fork();
+    if (pid < 0) {
+      _perr("fork");
+      _exit(-1);
+    }
+
+    if (pid == 0) {			/* child process */
+      /* Run the command. */
+      if (execlp(staprun, basename (staprun), "-d", modname, NULL) < 0) {
+	if (errno == ENOEXEC) {
+	  char *cmd;
+	  if (asprintf(&cmd, "%s -d '%s'", staprun, modname) > 0)
+	    execl("/bin/sh", "sh", "-c", cmd, NULL);
+	  free(cmd);
+	}
+	perror(staprun);
+	_exit(1);
+      }
+    }
+
+    /* parent process */
+    if (waitpid(pid, &rstatus, 0) < 0) {
+      _perr("waitpid");
+      _exit(-1);
+    }
+
+    if (WIFEXITED(rstatus)) {
+      _exit(WEXITSTATUS(rstatus));
+    }
+    _exit(-1);
+#endif
   }
   _exit(0);
 }
@@ -401,21 +489,14 @@ int stp_main_loop(void)
     switch (type) {
 #ifdef STP_OLD_TRANSPORT
     case STP_REALTIME_DATA:
-      {
-        ssize_t bw = write(out_fd[0], data, nb);
-        if (bw >= 0 && bw != nb) {
-          nb = nb - bw;
-          bw = write(out_fd[0], data, nb);
-        }
-        if (bw != nb) {
-          _perr("write error (nb=%ld)", (long)nb);
-          cleanup_and_exit(0);
-        }
-        break;
+      if (write_realtime_data(data, nb)) {
+        _perr("write error (nb=%ld)", (long)nb);
+        cleanup_and_exit(0);
       }
+      break;
 #endif
     case STP_OOB_DATA:
-      fputs((char *)data, stderr);
+      eprintf("%s", (char *)data);
       break;
     case STP_EXIT:
       {
@@ -424,6 +505,14 @@ int stp_main_loop(void)
         cleanup_and_exit(0);
         break;
       }
+    case STP_REQUEST_EXIT:
+      {
+        /* module asks us to start exiting, so send STP_EXIT */
+        dbug(2, "got STP_REQUEST_EXIT\n");
+        int32_t rc, btype = STP_EXIT;
+        rc = write(control_channel, &btype, sizeof(btype));
+        break;
+      }
     case STP_START:
       {
         struct _stp_msg_start *t = (struct _stp_msg_start *)data;
