@@ -30,6 +30,7 @@ struct functiondecl;
 struct derived_probe;
 struct be_derived_probe_group;
 struct dwarf_derived_probe_group;
+struct kprobe_derived_probe_group;
 struct uprobe_derived_probe_group;
 struct utrace_derived_probe_group;
 struct itrace_derived_probe_group;
@@ -37,6 +38,7 @@ struct task_finder_derived_probe_group;
 struct timer_derived_probe_group;
 struct profile_derived_probe_group;
 struct mark_derived_probe_group;
+struct tracepoint_derived_probe_group;
 struct hrtimer_derived_probe_group;
 struct perfmon_derived_probe_group;
 struct procfs_derived_probe_group;
@@ -73,8 +75,9 @@ struct statistic_decl
 struct systemtap_session
 {
   systemtap_session ();
-  // NB: new POD members likely need to be explicitly cleared in the ctor.
-  // See elaborate.cxx.
+  // NB: It is very important for all of the above (and below) fields
+  // to be cleared in the systemtap_session ctor (elaborate.cxx)
+  // and/or main.cxx(main).
 
   // command line args
   std::vector<std::string> include_path;
@@ -89,6 +92,7 @@ struct systemtap_session
   std::string module_name;
   std::string stapconf_name;
   std::string output_file;
+  std::string size_option;
   std::string cmd;
   int target_pid;
   int last_pass;
@@ -110,11 +114,16 @@ struct systemtap_session
   bool need_uprobes;
   bool load_only; // flight recorder mode
 
+  // NB: It is very important for all of the above (and below) fields
+  // to be cleared in the systemtap_session ctor (elaborate.cxx)
+  // and/or main.cxx(main).
+
   // Cache data
   bool use_cache;
   std::string cache_path;
   std::string hash_path;
   std::string stapconf_path;
+  std::string tracequery_path;
 
   // dwarfless operation
   bool consult_symtab;
@@ -122,6 +131,13 @@ struct systemtap_session
   bool ignore_vmlinux;
   bool ignore_dwarf;
 
+  // Skip bad $ vars
+  bool skip_badvars;
+
+  // NB: It is very important for all of the above (and below) fields
+  // to be cleared in the systemtap_session ctor (elaborate.cxx)
+  // and/or main.cxx(main).
+
   // temporary directory for module builds etc.
   // hazardous - it is "rm -rf"'d at exit
   std::string tmpdir;
@@ -155,6 +171,7 @@ struct systemtap_session
   // session.probes vector.
   be_derived_probe_group* be_derived_probes;
   dwarf_derived_probe_group* dwarf_derived_probes;
+  kprobe_derived_probe_group* kprobe_derived_probes;
   uprobe_derived_probe_group* uprobe_derived_probes;
   utrace_derived_probe_group* utrace_derived_probes;
   itrace_derived_probe_group* itrace_derived_probes;
@@ -162,10 +179,15 @@ struct systemtap_session
   timer_derived_probe_group* timer_derived_probes;
   profile_derived_probe_group* profile_derived_probes;
   mark_derived_probe_group* mark_derived_probes;
+  tracepoint_derived_probe_group* tracepoint_derived_probes;
   hrtimer_derived_probe_group* hrtimer_derived_probes;
   perfmon_derived_probe_group* perfmon_derived_probes;
   procfs_derived_probe_group* procfs_derived_probes;
 
+  // NB: It is very important for all of the above (and below) fields
+  // to be cleared in the systemtap_session ctor (elaborate.cxx)
+  // and/or main.cxx(main).
+
   // unparser data
   translator_output* op;
   unparser* up;
@@ -180,6 +202,10 @@ struct systemtap_session
   std::set<std::string> unwindsym_modules;
   struct module_cache* module_cache;
 
+  // NB: It is very important for all of the above (and below) fields
+  // to be cleared in the systemtap_session ctor (elaborate.cxx)
+  // and/or main.cxx(main).
+
   std::set<std::string> seen_errors;
   std::set<std::string> seen_warnings;
   unsigned num_errors () { return seen_errors.size(); }
@@ -191,7 +217,9 @@ struct systemtap_session
   void print_error_source (std::ostream&, std::string&, const token* tok);
   void print_warning (const std::string& w, const token* tok = 0);
 
-  // reNB: new POD members likely need to be explicitly cleared in the ctor.
+  // NB: It is very important for all of the above (and below) fields
+  // to be cleared in the systemtap_session ctor (elaborate.cxx)
+  // and/or main.cxx(main).
 };
 
 
